#!/usr/bin/perl
#$Id: ajax_furnace.cgi,v 1.27 2020/08/24 06:39:11 sqko Exp $
use strict;
use warnings;
use URI::Escape;
use RFC::Header;
use RFC::Main;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Rig;
use RFC::Furnace;
use CGI qw(:standard :netscape);
use LWP::UserAgent;
my $ajax = 1;
$ajax = 0 if (param('noajax'));




my $f = get_CGI_value_clean('furnace');
my $run = get_CGI_value_clean('runnr');
unless ($f)
{
    $f = get_CGI_value_clean('rig');
}
my $ac = get_CGI_value_clean('action');
unless ($ac)
{
    print header(-access_control_allow_origin => '*',-content_type=>'text/plain',-charset=>'utf-8');
    print <<END;

Oops.. no parameters, displaying interface definitions!

Common parameters:

Parameter 'action', determines which function to call (each described below)
Generally, the script returns OK on the first line if input is valid, and an errorstring if not.
Thus remember to check for the 'OK' string as part of the ajax response function.

If password protection is implemented the following additional parameters may be nescesarry for most of the functions:
'name' Username, text
'pass' Session key, text

Action 'is_ip_ok'
Returns a string describing which public IP the remote system is using (in order to fix proxy issues)
as well as if that IP is one of the certified addresses.
Called with no parameters, and do NOT require a valid session.

Action 'status'
Parameters:
'furnace'  Rig number, integer
Returns the status of the specified furnace. Do not require a valis session.

Action 'status_user'
Parameters:
'furnace'  Rig number, integer
Returns the status of the specified furnace.
If a furnace run is active, the following informatin is also returned on separate lines:
The username of the user starting the furnace run.
The expected total runtime in hours
The currently elapsed time of the furnace run.
Do not require a valis session.
If no furnace run is running the last three lines will be empty


Action 'runnr'
Parameters:
'furnace'  Rig number, integer
Returns the current runnumber for the specified furnace.
Checks all servers in cluster.

Action 'furnaces':
No parameters
Displays a list of possible rigs (all in cluster).


Action 'is_furnace':
Parameters:
'furnace'  Rig number to check if it exists, integer

Action 'get_server':
parameters:
'furnace'  Rig number to return actual server name for if it exists, integer

Action 'is_run':
Parameters:
'furnace'  Rig number, integer
'runnr'    Test number to check, integer
Checks if the runnumber specified is valid for the rig in question on any of the servers on the cluster.
Checks all servers in cluster.


Action 'run':
Parameters:
'furnace'  Rig number, integer
'runnr'    Test number, integer
Returns the temperature, ramprate and dwelltime for the specified run.
Checks all servers in cluster.


Action 'total_runs':
No parameters
Returns statistics on all furnace runs for all furnaces on all servers.
It returns the total number of successfull furnace runs as will as the time of the first and the time of the last.
A succesfull furnace run is defined as onw where the run time is more than 1 hour.

Action 'run_info':
Parameters:
'furnace'  Rig number, integer
'runnr'    Test number, integer
Returns the maximum temperature, the total runtime, the content of the info file as well as the proglog for the specified furnace run.
Checks all servers in cluster.

Action 'equipid'
Parameters:
'furnace'  Rig number, integer
Returns the equipment id (if specified in the furnace configuration) for the furnace specified.
Checks all servers in cluster.

END
    exit;
}


print header(-Access_Control_Allow_Origin => '*');

if ($ac eq 'status')
{
    my $rig = RFC::Rig->new($f);
    my $status = $rig->get_cv('main','status');
    if ($status && $status !~ /^OK/i)
    {
	print $status;
	exit;
    }
    my $fn = '/home/http/html/rig'.$f.'/current_program.txt';
    if (-e $fn)
    {
	print 'Running';
    }
    else
    {
	print 'Free';
    }
    exit;
}


if ($ac eq 'status_user')
{
    my $rig = RFC::Rig->new($f);
    my $status = $rig->get_cv('main','status');
    if ($status && $status !~ /^OK/i)
    {
	print join("\n",$status,'','','');
	exit;
    }
    my $r = RFC::Furnace->new($f);
    my %h = $r->get_run_info();
    my $user = $h{'ordered_by'};
    $user = $h{'started_by'} unless ($user);
    my $fn = '/home/http/html/rig'.$f.'/current_program.txt';
    if (-e $fn)
    {
	$fn = '/home/http/html/rig'.$f.'/furnace_runs/'.$r->runnr().'/Program.txt';
	open MYIN,'<',$fn or die "Could not open file $fn for reading $!\n";
	my @l = <MYIN>;
	my $str = $l[-1];
	my $total = 0;
	if ($str && $str =~ /\=\s*([\.\d]+)/)
	{
	    $total = $1
	};
	my $t = sprintf("%.2f",$r->get_runtime());
	close MYIN;
	print join("\n",'Running',$user,$total,$t);
    }
    else
    {
	print join("\n",'Free','','','');
    }
    exit;
}

if ($ac eq 'is_ip_ok')
{
    my $ok = &RFC::Furnace::is_authorised();
    my $ip = $ENV{'REMOTE_ADDR'};
    print "Your IP is: $ip";
    if ($ok)
    {
	print "And you can access data on this system without credentials!\n";
    }
    else
    {
	print "And you need a valid session to access most data on this system!\n";
    }
}

### Below requires valid session #



my $uid = login_ok();
unless ($uid || &RFC::Furnace::is_authorised())
{       
    print "Access denied!\n";
    exit;
}
my $cred = '&name='.&get_user().'&pass='.uri_escape(&get_pass());

if ($ac eq 'runnr_local')
{
    my $rig = RFC::Furnace->new($f);
    print "OK\n";
    my $run = $rig->runnr();
    my $fn = '/home/http/html/rig'.$f.'/current_program.txt';
    $run-- if (-e $fn);
    print $run;
    exit;
}



my @list = ();
my $ssl = 0;
$ssl = 1 if (CGI::https());
my $listserver = get_cv('servers','listserver');
if ($listserver)
{
    chomp $listserver;
    my $str = &RFC::Main::report_client($listserver,'servers');
    my @l = split(/\t/,$str) if ($str && $str !~ /ERROR/);
    foreach (@l)
    {
        if (/([\w\.\-]+)/)
        {
            push @list,$1;
        }
    }
}

if ($ac eq 'furnace_info')
{
    my $min = 0;
    my $max = 0;
    my $count = 0;
    my $r = RFC::Furnace->new($f);
    for (my $x = 0; $x < $r->runnr; $x++)
    {
	my @lines = $r->get_data($x);
	next unless (@lines);
	my @l = split(/\s/,$lines[0]);
	my $start = $l[2];
	@l = split(/\s/,$lines[-1]);
	my $end = $l[2];
	next unless ($start && $end);
	### Only include runs longer than 1 hour ###
	next unless (($end - $start) > 3600);
	$count++;
	$min = $start if ($min == 0 || $start < $min);
	$max = $end if ($max == 0 || $max < $end);
    }
    print join("\n",'OK',$count,$min,$max);
    exit;
}

if ($ac eq 'total_runs')
{
    my $sum = 0;
    my $mindate = 0;
    my $maxdate = 0;
    foreach my $s (@list)
    {
        my $str = &RFC::Main::report_client($s,'rigs');
        if (defined($str))
        {
            chomp $str;
            chomp $str;
            chomp $str;
            my @l = split(/\t/,$str);
            foreach my $r (@l)
            {
                my $ua = LWP::UserAgent->new();
                $ua->timeout(60);
                my $url = 'http://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=furnace_info&furnace=' . $r . $cred;
                $url = 'https://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=furnace_info&furnace=' . $r . $cred if ($ssl);
                my $request = new HTTP::Request('GET',$url);
                my $response = $ua->request($request);
                my $content = $response->content();
		next unless ($content);
		my ($ok,$count,$min,$max) = split(/\n/,$content);
		next unless ($ok eq 'OK');
		$sum += $count;
		$mindate = $min if ($mindate == 0 || $min < $mindate);
		$maxdate = $max if ($maxdate == 0 || $max > $maxdate);
            }
        }
    }
    my @start = localtime($mindate);
    my @end = localtime($maxdate);
    $mindate = sprintf("%04i-%02i-%02i",$start[5]+1900,$start[4]+1,$start[3]);
    $maxdate = sprintf("%04i-%02i-%02i",$end[5]+1900,$end[4]+1,$end[3]);
    print join("\n",'OK',$sum,$mindate,$maxdate),"\n";
    exit;
}

if ($ac eq 'furnaces')
{
    print "OK\n";
    my @rigs = ();
    foreach my $s (@list)
    {
        my $str = &RFC::Main::report_client($s,'rigs');
        if (defined($str))
        {
            chomp $str;
            chomp $str;
            chomp $str;
            push @rigs,split(/\t/,$str);
        }
    }
    print join("\n",(sort {$a <=> $b} @rigs)),"\n";
    exit;
}

if ($ac eq 'is_furnace')
{
    foreach my $s (@list)
    {
        my $str = &RFC::Main::report_client($s,'rigs');
        if (defined($str))
        {
            chomp $str;
            chomp $str;
            chomp $str;
            my @l = split(/\t/,$str);
            foreach (@l)
            {
                next unless ($_ == $f);
                print "OK\n";
                print "1\n";
                exit;
            }
        }    
    }
    print "Warning\n";
    print "No furnace found\n";
    exit;
}

if ($ac eq 'get_server')
{
    foreach my $s (@list)
    {
        my $str = &RFC::Main::report_client($s,'rigs');
        if (defined($str))
        {
            chomp $str;
            chomp $str;
            chomp $str;
            my @l = split(/\t/,$str);
            foreach (@l)
            {
                next unless ($_ == $f);
                print "OK\n";
                print "$s\n";
                exit;
            }
        }    
    }
    print "Warning\n";
    print "No furnace found\n";
    exit;
}


if ($ac eq 'active_furnace')
{
    foreach my $s (@list)
    {
        my $str = &RFC::Main::report_client($s,'active_rigs');
        if (defined($str))
        {
            chomp $str;
            chomp $str;
            chomp $str;
            my @l = split(/\t/,$str);
            foreach (@l)
            {
                next unless ($_ == $f);
                print "OK\n";
                print "1\n";
                exit;
            }
        }    
    }
    print "Warning\n";
    print "No furnace found\n";
    exit;
}

if ($ac eq 'runnr')
{
    my $res = "Warning\nNo furnace found!\n";
  OUTER:
    foreach my $s (@list)
    {
        my $str = &RFC::Main::report_client($s,'rigs');
        if (defined($str))
        {
            chomp $str;
            chomp $str;
            chomp $str;
            my @l = split(/\t/,$str);
            foreach my $r (@l)
            {
                next unless ($f == $r);
                my $ua = LWP::UserAgent->new();
                $ua->timeout(60);
                my $url = 'http://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=runnr_local&furnace=' . $r . $cred;
                $url = 'https://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=runnr_local&furnace=' . $r . $cred if ($ssl);
                my $request = new HTTP::Request('GET',$url);
                my $response = $ua->request($request);
                my $content = $response->content();
                $res = $content if ($content);
                last OUTER;
            }
        }
    }
    print $res;
    exit;
}

if ($ac eq 'equipid')
{
    my $res = "Warning\nNo furnace found!\n";
  OUTER:
    foreach my $s (@list)
    {
        my $str = &RFC::Main::report_client($s,'rigs');
        if (defined($str))
        {
            chomp $str;
            chomp $str;
            chomp $str;
            my @l = split(/\t/,$str);
            foreach my $r (@l)
            {
                next unless ($f == $r);
                my $ua = LWP::UserAgent->new();
                $ua->timeout(60);
                my $url = 'http://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=equipid_local&furnace=' . $r . $cred;
                $url = 'https://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=equipid_local&furnace=' . $r . $cred if ($ssl);
                my $request = new HTTP::Request('GET',$url);
                my $response = $ua->request($request);
                my $content = $response->content();
                $res = $content if ($content);
                last OUTER;
            }
        }
    }
    print $res;
    exit;
}

if ($ac eq 'is_run')
{
    my $res = "Warning\nNo furnace found!\n";
  OUTER:
    foreach my $s (@list)
    {
        my $str = &RFC::Main::report_client($s,'rigs');
        if (defined($str))
        {
            chomp $str;
            chomp $str;
            chomp $str;
            my @l = split(/\t/,$str);
            foreach my $r (@l)
            {
                next unless ($f == $r);
                my $ua = LWP::UserAgent->new();
                $ua->timeout(60);
                my $url = 'http://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=rig_session&furnace=' . $r . '&runnr=' . $run . $cred;
                $url = 'https://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=rig_session&furnace=' . $r . '&runnr=' . $run . $cred if ($ssl);
                my $request = new HTTP::Request('GET',$url);
                my $response = $ua->request($request);
                my $content = $response->content();
                $res = $content if ($content);
                last OUTER;
            }
        }
    }
    print $res;
    exit;
}

if ($ac eq 'run')
{
    my $res = "Warning\nNo furnace found!\n";
  OUTER:
    foreach my $s (@list)
    {
        my $str = &RFC::Main::report_client($s,'rigs');
        if (defined($str))
        {
            chomp $str;
            chomp $str;
            chomp $str;
            my @l = split(/\t/,$str);
            foreach my $r (@l)
            {
                next unless ($f == $r);
                my $ua = LWP::UserAgent->new();
                $ua->timeout(60);
                my $url = 'http://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=run_local&furnace=' . $r . '&runnr=' . $run . $cred;
                $url = 'https://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=run_local&furnace=' . $r . '&runnr=' . $run . $cred if ($ssl);
                my $request = new HTTP::Request('GET',$url);
                my $response = $ua->request($request);
                my $content = $response->content();
                $res = $content if ($content);
                last OUTER;
            }
        }
    }
    print $res;
    exit;
}


if ($ac eq 'run_info')
{
    my $res = "Warning\nNo furnace found!\n";
  OUTER:
    foreach my $s (@list)
    {
        my $str = &RFC::Main::report_client($s,'rigs');
        if (defined($str))
        {
            chomp $str;
            chomp $str;
            chomp $str;
            my @l = split(/\t/,$str);
            foreach my $r (@l)
            {
                next unless ($f == $r);
                my $ua = LWP::UserAgent->new();
                $ua->timeout(60);
                my $url = 'http://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=run_info_local&furnace=' . $r . '&runnr=' . $run . $cred;
                $url = 'https://'.$s . '/cgi-bin/RFCfurnace/ajax_furnace.cgi?action=run_info_local&furnace=' . $r . '&runnr=' . $run . $cred if ($ssl);
                my $request = new HTTP::Request('GET',$url);
                my $response = $ua->request($request);
                my $content = $response->content();
                $res = $content if ($content);
                last OUTER;
            }
        }
    }
    print $res;
    exit;
}


if ($ac eq 'rig_session')
{
    my $r = RFC::Furnace->new($f);
    my $session = $r->runnr();
    if ($run > 0 && $run <= $session)
    {
        print "OK\n" if ($ajax);
        print "1\n";
        exit;   
    }
    else
    {
        print "Warning\n";
        print "Run $run for $r not valid!\n";
    }
    exit;
}


if ($ac eq 'is_session')
{
    my $r = RFC::Furnace->new($f);
    my @l = $r->get_lines($r->append_program($r->get_program($run)));
    print "OK\n";
    if (@l && $l[0])
    {
	print $l[0],"\n"; 
    }
    else
    {
	print "0\n";
    }
    exit;
}    

if ($ac eq 'run_local')
{
    my $r = RFC::Furnace->new($f);
    my @l = $r->get_lines($r->append_program($r->get_program($run)));
    my $lines = shift(@l);
    print "OK\n";
    print join("\n",@l),"\n";
    print "Warning: Not a simple furnace run, more than 3 steps in program. Steps detected: $lines\n" unless ($lines == 3);
    exit;
}


if ($ac eq 'run_info_local')
{
    my $r = RFC::Furnace->new($f);
    my @l = $r->get_program($run);
    my $max = -273;
    foreach (@l)
    {
	next unless /^temp/;
	/([\-\d\.]+)/;
	my $t = $1;
	$max = $t if ($t > $max);
    }
    if ($l[-1] =~ /Total/)
    {
	my $totalt = $r->get_runtime($run);
	print "OK\n";
	print "Maxtemp: $max\n";
	printf("Total time = %.02f\n",$totalt);
	my @info = $r->get_info($run);
	print join("\n",@info);
	my @proglog = $r->get_proglog($run);
	print "\n\nProgram log:\n";
	print join("\n",@proglog);
    }
}


if ($ac eq 'equipid_local')
{
    my $r = RFC::Rig->new($f);
    my $equipid = $r->get_cv('main','equipmentid');
    $equipid = 0 unless ($equipid);
    print "OK\n$equipid";
}
